package xyz.brassgoggledcoders.boilerplate.lib.common.entity;

import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.projectile.EntityThrowable;
import net.minecraft.init.Blocks;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.util.MathHelper;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.util.Vec3;
import net.minecraft.world.World;

import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

/**
 * @author Surseance (Johnny Eatmon)
 *
 */
public abstract class BaseThrowableEntity extends EntityThrowable
{
	private int xTile = -1;
	private int yTile = -1;
	private int zTile = -1;
	private Block inTile;
	protected boolean inGround;
	public int throwableShake;
	/** The entity that threw this throwable item. */
	private EntityLivingBase thrower;
	private String throwerName;
	private int ticksInGround;
	private int ticksInAir;

	public BaseThrowableEntity(World p_i1776_1_)
	{
		super(p_i1776_1_);
		this.func_70105_a(0.25F, 0.25F);
	}

	/**
	 * Checks if the entity is in range to render by using the past in distance
	 * and comparing it to its average edge length * 64 * renderDistanceWeight
	 * Args: distance
	 */
	@Override
	@SideOnly(Side.CLIENT)
	public boolean func_70112_a(double p_70112_1_)
	{
		double d1 = this.func_174813_aQ().func_72320_b() * 4.0D;
		d1 *= 64.0D;
		return p_70112_1_ < (d1 * d1);
	}

	public BaseThrowableEntity(World world, EntityLivingBase entity)
	{
		super(world, entity);
	}

	public BaseThrowableEntity(World p_i1778_1_, double p_i1778_2_, double p_i1778_4_, double p_i1778_6_)
	{
		super(p_i1778_1_, p_i1778_2_, p_i1778_4_, p_i1778_6_);
	}

	/*
	 * TODO What is this?
	 * 
	 * @Override protected float func_70182_d() { return 1.5F; }
	 * 
	 * @Override protected float func_70183_g() { return 0.0F; }
	 */

	/**
	 * Similar to setArrowHeading, it's point the throwable entity to a x, y, z
	 * direction.
	 */
	@Override
	public void func_70186_c(double p_70186_1_, double p_70186_3_, double p_70186_5_, float p_70186_7_, float p_70186_8_)
	{
		float f2 = MathHelper.func_76133_a((p_70186_1_ * p_70186_1_) + (p_70186_3_ * p_70186_3_) + (p_70186_5_ * p_70186_5_));
		p_70186_1_ /= f2;
		p_70186_3_ /= f2;
		p_70186_5_ /= f2;
		p_70186_1_ += this.field_70146_Z.nextGaussian() * 0.007499999832361937D * p_70186_8_;
		p_70186_3_ += this.field_70146_Z.nextGaussian() * 0.007499999832361937D * p_70186_8_;
		p_70186_5_ += this.field_70146_Z.nextGaussian() * 0.007499999832361937D * p_70186_8_;
		p_70186_1_ *= p_70186_7_;
		p_70186_3_ *= p_70186_7_;
		p_70186_5_ *= p_70186_7_;
		this.field_70159_w = p_70186_1_;
		this.field_70181_x = p_70186_3_;
		this.field_70179_y = p_70186_5_;
		float f3 = MathHelper.func_76133_a((p_70186_1_ * p_70186_1_) + (p_70186_5_ * p_70186_5_));
		this.field_70126_B = this.field_70177_z = (float) ((Math.atan2(p_70186_1_, p_70186_5_) * 180.0D) / Math.PI);
		this.field_70127_C = this.field_70125_A = (float) ((Math.atan2(p_70186_3_, f3) * 180.0D) / Math.PI);
		this.ticksInGround = 0;
	}

	/**
	 * Sets the velocity to the args. Args: x, y, z
	 */
	@Override
	@SideOnly(Side.CLIENT)
	public void func_70016_h(double p_70016_1_, double p_70016_3_, double p_70016_5_)
	{
		this.field_70159_w = p_70016_1_;
		this.field_70181_x = p_70016_3_;
		this.field_70179_y = p_70016_5_;

		if ((this.field_70127_C == 0.0F) && (this.field_70126_B == 0.0F))
		{
			float f = MathHelper.func_76133_a((p_70016_1_ * p_70016_1_) + (p_70016_5_ * p_70016_5_));
			this.field_70126_B = this.field_70177_z = (float) ((Math.atan2(p_70016_1_, p_70016_5_) * 180.0D) / Math.PI);
			this.field_70127_C = this.field_70125_A = (float) ((Math.atan2(p_70016_3_, f) * 180.0D) / Math.PI);
		}
	}

	/**
	 * Called to update the entity's position/logic.
	 */
	@Override
	public void func_70071_h_()
	{
		this.field_70142_S = this.field_70165_t;
		this.field_70137_T = this.field_70163_u;
		this.field_70136_U = this.field_70161_v;
		super.func_70071_h_();

		if (this.throwableShake > 0)
		{
			--this.throwableShake;
		}

		if (this.inGround)
		{
			if (this.field_70170_p.func_180495_p(this.func_180425_c()).func_177230_c() == this.inTile)
			{
				++this.ticksInGround;

				if (this.ticksInGround == 1200)
				{
					this.func_70106_y();
				}

				return;
			}

			this.inGround = false;
			this.field_70159_w *= this.field_70146_Z.nextFloat() * 0.2F;
			this.field_70181_x *= this.field_70146_Z.nextFloat() * 0.2F;
			this.field_70179_y *= this.field_70146_Z.nextFloat() * 0.2F;
			this.ticksInGround = 0;
			this.ticksInAir = 0;
		}
		else
		{
			++this.ticksInAir;
		}

		Vec3 vec3 = new Vec3(this.field_70165_t, this.field_70163_u, this.field_70161_v);
		Vec3 vec31 = new Vec3(this.field_70165_t + this.field_70159_w, this.field_70163_u + this.field_70181_x, this.field_70161_v + this.field_70179_y);
		MovingObjectPosition movingobjectposition = this.field_70170_p.func_72933_a(vec3, vec31);
		vec3 = new Vec3(this.field_70165_t, this.field_70163_u, this.field_70161_v);
		vec31 = new Vec3(this.field_70165_t + this.field_70159_w, this.field_70163_u + this.field_70181_x, this.field_70161_v + this.field_70179_y);

		if (movingobjectposition != null)
		{
			vec31 = new Vec3(movingobjectposition.field_72307_f.field_72450_a, movingobjectposition.field_72307_f.field_72448_b, movingobjectposition.field_72307_f.field_72449_c);
		}

		if (!this.field_70170_p.field_72995_K)
		{
			Entity entity = null;
			List list = this.field_70170_p.func_72839_b(this,
					this.func_174813_aQ().func_72321_a(this.field_70159_w, this.field_70181_x, this.field_70179_y).func_72314_b(1.0D, 1.0D, 1.0D));
			double d0 = 0.0D;
			EntityLivingBase entitylivingbase = this.func_85052_h();

			for (Object obj : list)
			{
				Entity entity1 = (Entity) obj;

				if (entity1.func_70067_L() && ((entity1 != entitylivingbase) || (this.ticksInAir >= 5)))
				{
					float f = 0.3F;
					AxisAlignedBB axisalignedbb = entity1.func_174813_aQ().func_72314_b(f, f, f);
					MovingObjectPosition movingobjectposition1 = axisalignedbb.func_72327_a(vec3, vec31);

					if (movingobjectposition1 != null)
					{
						double d1 = vec3.func_72438_d(movingobjectposition1.field_72307_f);

						if ((d1 < d0) || (d0 == 0.0D))
						{
							entity = entity1;
							d0 = d1;
						}
					}
				}
			}

			if (entity != null)
			{
				movingobjectposition = new MovingObjectPosition(entity);
			}
		}

		if (movingobjectposition != null)
		{
			if ((movingobjectposition.field_72313_a == MovingObjectPosition.MovingObjectType.BLOCK)
					&& (this.field_70170_p.func_180495_p(movingobjectposition.func_178782_a()).func_177230_c() == Blocks.field_150427_aO))
			{
				this.field_71087_bX = true;
			}
			else
			{
				this.func_70184_a(movingobjectposition);
			}
		}

		this.field_70165_t += this.field_70159_w;
		this.field_70163_u += this.field_70181_x;
		this.field_70161_v += this.field_70179_y;
		float f1 = MathHelper.func_76133_a((this.field_70159_w * this.field_70159_w) + (this.field_70179_y * this.field_70179_y));
		this.field_70177_z = (float) ((Math.atan2(this.field_70159_w, this.field_70179_y) * 180.0D) / Math.PI);

		for (this.field_70125_A = (float) ((Math.atan2(this.field_70181_x, f1) * 180.0D) / Math.PI); (this.field_70125_A
				- this.field_70127_C) < -180.0F; this.field_70127_C -= 360.0F)
		{
			;
		}

		while ((this.field_70125_A - this.field_70127_C) >= 180.0F)
		{
			this.field_70127_C += 360.0F;
		}

		while ((this.field_70177_z - this.field_70126_B) < -180.0F)
		{
			this.field_70126_B -= 360.0F;
		}

		while ((this.field_70177_z - this.field_70126_B) >= 180.0F)
		{
			this.field_70126_B += 360.0F;
		}

		this.field_70125_A = this.field_70127_C + ((this.field_70125_A - this.field_70127_C) * 0.2F);
		this.field_70177_z = this.field_70126_B + ((this.field_70177_z - this.field_70126_B) * 0.2F);
		float f2 = 0.99F;
		float f3 = this.func_70185_h();

		if (this.func_70090_H())
		{
			for (int i = 0; i < 4; ++i)
			{
				float f4 = 0.25F;
				this.field_70170_p.func_175688_a(EnumParticleTypes.WATER_BUBBLE, this.field_70165_t - (this.field_70159_w * f4), this.field_70163_u - (this.field_70181_x * f4),
						this.field_70161_v - (this.field_70179_y * f4), this.field_70159_w, this.field_70181_x, this.field_70179_y);
			}

			f2 = 0.8F;
		}

		this.field_70159_w *= f2;
		this.field_70181_x *= f2;
		this.field_70179_y *= f2;
		this.field_70181_x -= f3;
		this.func_70107_b(this.field_70165_t, this.field_70163_u, this.field_70161_v);
	}

	/**
	 * (abstract) Protected helper method to write subclass entity data to NBT.
	 */
	@Override
	public void func_70014_b(NBTTagCompound p_70014_1_)
	{
		p_70014_1_.func_74777_a("xTile", (short) this.xTile);
		p_70014_1_.func_74777_a("yTile", (short) this.yTile);
		p_70014_1_.func_74777_a("zTile", (short) this.zTile);
		p_70014_1_.func_74774_a("inTile", (byte) Block.func_149682_b(this.inTile));
		p_70014_1_.func_74774_a("shake", (byte) this.throwableShake);
		p_70014_1_.func_74774_a("inGround", (byte) (this.inGround ? 1 : 0));

		if (((this.throwerName == null) || (this.throwerName.length() == 0)) && (this.thrower != null) && (this.thrower instanceof EntityPlayer))
		{
			this.throwerName = this.thrower.func_174793_f().func_70005_c_();
		}

		p_70014_1_.func_74778_a("ownerName", this.throwerName == null ? "" : this.throwerName);
	}

	/**
	 * (abstract) Protected helper method to read subclass entity data from NBT.
	 */
	@Override
	public void func_70037_a(NBTTagCompound p_70037_1_)
	{
		this.xTile = p_70037_1_.func_74765_d("xTile");
		this.yTile = p_70037_1_.func_74765_d("yTile");
		this.zTile = p_70037_1_.func_74765_d("zTile");
		this.inTile = Block.func_149729_e(p_70037_1_.func_74771_c("inTile") & 255);
		this.throwableShake = p_70037_1_.func_74771_c("shake") & 255;
		this.inGround = p_70037_1_.func_74771_c("inGround") == 1;
		this.throwerName = p_70037_1_.func_74779_i("ownerName");

		if ((this.throwerName != null) && (this.throwerName.length() == 0))
		{
			this.throwerName = null;
		}
	}

	/*
	 * TODO: Shadows
	 * 
	 * @Override
	 * 
	 * @SideOnly(Side.CLIENT) public float getShadowSize() { return 0.0F; }
	 */

	@Override
	public EntityLivingBase func_85052_h()
	{
		if ((this.thrower == null) && (this.throwerName != null) && (this.throwerName.length() > 0))
		{
			this.thrower = this.field_70170_p.func_72924_a(this.throwerName);
		}

		return this.thrower;
	}
}
